<?php

namespace Drupal\toolshed_media\Form;

use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Manage the image styles available for embedded media.
 */
class MediaEmbedStylesForm extends ConfigFormBase {

  /**
   * Entity storage handler for image styles.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $imageStyleStorage;

  /**
   * Create a new instance of the MediaEmbedStylesForm.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct($config_factory);

    $this->imageStyleStorage = $entity_type_manager->getStorage('image_style');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'toolshed_media_embed_styles_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['toolshed.media.embed_styles'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $defaults = $this->config('toolshed.media.embed_styles');

    $form['filter_method'] = [
      '#type' => 'radios',
      '#title' => $this->t('Styles filtering'),
      '#required' => TRUE,
      '#options' => [
        'only_listed' => $this->t('Include only the selected styles'),
        'exclude_listed' => $this->t('Exclude the selected styles'),
      ],
      '#default_value' => $defaults->get('filter_method') ?: 'only_listed',
    ];

    $styleOpts = ['none' => $this->t('Original')];
    foreach ($this->imageStyleStorage->loadMultiple() as $style) {
      $styleOpts[$style->id()] = $style->label();
    }

    $form['styles'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Image styles available'),
      '#options' => $styleOpts,
      '#default_value' => $defaults->get('styles') ?: [],
    ];

    $form['actions'] = [
      '#type' => 'actions',

      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Save Settings'),
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    $settings = $this->config('toolshed.media.embed_styles');
    $settings->set('filter_method', $values['filter_method']);
    $settings->set('styles', array_filter($values['styles']));
    $settings->save();
  }

}
