<?php

namespace Drupal\toolshed_media\Plugin\Toolshed\ThirdPartyConfig;

use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\Entity\ConfigEntityInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\toolshed\Plugin\ThirdPartyConfigBase;

/**
 * Add toolshed media third party settings to the media_type edit form.
 *
 * @ToolshedThirdPartyConfig(
 *   id = "toolshed_media_behaviors",
 *   label = @Translation("Page display behaviors"),
 *   entity_types = {
 *     "media_type"
 *   },
 * )
 */
class MediaThirdPartyConfig extends ThirdPartyConfigBase implements ContainerFactoryPluginInterface {

  /**
   * The current media settings.
   *
   * @var \Drupal\Core\Config\ImmutableConfigInterface
   */
  protected $mediaSettings;

  /**
   * Create a new instance of the MediaThirdPartyConfig plugin.
   *
   * @param array $configuration
   *   The plugin configuration.
   * @param string $plugin_id
   *   The plugin ID.
   * @param mixed $plugin_definition
   *   The plugin definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConfigFactoryInterface $config_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->mediaSettings = $config_factory->get('media.settings');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDefaultSettings(): array {
    return [
      'redirect_to_file' => FALSE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function formElements(ConfigEntityInterface $entity, array $settings, FormStateInterface $form_state): array {
    $elements = [
      '#type' => 'checkbox',
      '#title' => $this->t('Redirect non-admins to file'),
      '#default_value' => $settings['redirect_to_file'],
    ];

    // If media does not have standalone URL settings enabled, disable
    // the checkbox and add some help text point out that this option relies
    // on this media configuration option.
    if (!$this->mediaSettings->get('standalone_url')) {
      $elements['#disabled'] = TRUE;
      $elements['#description'] = $this->t('Media standalone URLs are required for file redirects to occur. Enable this functionality <a href="@url">here</a>.', [
        '@url' => Url::fromRoute('media.settings')->toString(),
      ]);
    }

    return [
      'redirect_to_file' => $elements,
    ];
  }

}
