<?php

/**
 * @file
 * Toolshed search Hooks and callback for views.
 */

/**
 * Implements hook_views_data_alter().
 */
function toolshed_search_views_data_alter(&$data) {
  $indexStorage = \Drupal::entityTypeManager()->getStorage('search_api_index');
  $fieldHelper = \Drupal::service('search_api.fields_helper');

  /** @var \Drupal\search_api\IndexInterface $index */
  foreach ($indexStorage->loadMultiple() as $index) {
    if (!$index->getEntityTypes()) {
      continue;
    }

    $table = &$data['search_api_index_' . $index->id()];
    $fields = $fieldHelper->filterForPropertyPath($index->getFields(), NULL, 'entity_bundle');

    foreach ($fields as $name => $field) {
      if (isset($table[$name])) {
        $table[$name]['filter'] = [
          'id' => 'toolshed_entity_bundle_filter',
        ];
      }
    }

    // Add a special handler for filtering including or excluding media.
    // The search_api_datasource is always available and always set to this key.
    $table['toolshed_include_type_filter'] = [
      'title' => t('Toggle datasource'),
      'help' => t('Creates an exposed checkbox to toggle a datasource.'),
      'filter' => [
        'id' => 'toolshed_datasource_filter',
        'real field' => 'search_api_datasource',
      ],
    ];
  }
}
