<?php

namespace Drupal\toolshed\Cache\Context;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Cache\Context\CacheContextInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Routing\RouteMatchInterface;

/**
 * Create a cache context for a user based on the {user} route parameter.
 *
 * Drupal core only provides the `user` cache context, which is always only
 * the current user. This context adds the ability to select user account
 * from a route as the cache context.
 *
 * Cache context ID: 'route.user'.
 */
class RouteUserCacheContext implements CacheContextInterface {

  /**
   * Information about a route, typically this is matching the current route.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * Create a new cache context for user accounts (by specific user).
   *
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   A route match object to get parameter and user information from.
   */
  public function __construct(RouteMatchInterface $route_match) {
    $this->routeMatch = $route_match;
  }

  /**
   * {@inheritdoc}
   */
  public static function getLabel() {
    return t('User by route parameter');
  }

  /**
   * {@inheritdoc}
   */
  public function getContext() {
    $account = $this->routeMatch->getParameter('user');

    if ($account instanceof AccountInterface && $account->isAuthenticated()) {
      return $account->id();
    }

    return 'user.none';
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheableMetadata() {
    return new CacheableMetadata();
  }

}
