<?php

namespace Drupal\toolshed\ContextProvider;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Plugin\Context\Context;
use Drupal\Core\Plugin\Context\ContextProviderInterface;
use Drupal\Core\Plugin\Context\EntityContext;
use Drupal\Core\Plugin\Context\EntityContextDefinition;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\user\Entity\UserInterface;

/**
 * Provides a context for a user entity from the route.
 */
class UserRouteContext implements ContextProviderInterface {

  use StringTranslationTrait;

  /**
   * The route match object.
   *
   * @var Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * Entity storage handler for <em>User</em> entities.
   *
   * @var Drupal\Core\Entity\EntityStorageInterface
   */
  protected $userStorage;

  /**
   * Constructs a UserRouteContext.
   *
   * @param Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match object.
   * @param Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager, for discovery and creation of entity type
   *   information, and definitions.
   */
  public function __construct(RouteMatchInterface $route_match, EntityTypeManagerInterface $entity_type_manager) {
    $this->routeMatch = $route_match;
    $this->userStorage = $entity_type_manager->getStorage('user');
  }

  /**
   * {@inheritdoc}
   */
  public function getRuntimeContexts(array $unqualified_context_ids) {
    $value = NULL;

    if ($account = $this->routeMatch->getParameter('user')) {
      if ($account instanceof AccountInterface && !($account instanceof UserInterface)) {
        $account = $this->userStorage->load($account->id());
      }

      $value = $account;
    }

    $cacheability = new CacheableMetadata();
    $cacheability->setCacheContexts(['route.user']);

    $contextDef = EntityContextDefinition::fromEntityTypeId('user')
      ->setLabel($this->t('User from route'))
      ->setRequired(FALSE);

    $context = new Context($contextDef, $value);
    $context->addCacheableDependency($cacheability);

    return ['user' => $context];
  }

  /**
   * {@inheritdoc}
   */
  public function getAvailableContexts() {
    $context = EntityContext::fromEntityTypeId('user', $this->t('User from route'));
    return ['user' => $context];
  }

}
