<?php

namespace Drupal\toolshed\Plugin;

use Drupal\Component\Plugin\PluginInspectionInterface;
use Drupal\Core\Config\Entity\ConfigEntityInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Interface for applying config entity third party setting form elements.
 */
interface ThirdPartyConfigInterface extends PluginInspectionInterface {

  /**
   * Determine if a specific instance of a ConfigEntity applies to this plugin.
   *
   * The isApplicable() method allows for finer grained settings that may or
   * may not be relevant to a configuration entity type.
   *
   * @param Drupal\Core\Config\Entity\ConfigEntityInterface $entity
   *   The configuration entity to test if settings plugin settings apply.
   * @param string $op
   *   The name of the operation being performed on the entity.
   *
   * @return bool
   *   Returns TRUE if this third party form settings apply to the provided
   *   $entity parameter. FALSE if the settings do not.
   */
  public function isApplicable(ConfigEntityInterface $entity, $op);

  /**
   * Get the default third party settings configured by this plugin.
   *
   * @return array
   *   Default values to apply to the third party settings from this plugin.
   */
  public function getDefaultSettings(): array;

  /**
   * Add the third party settings to an entity form.
   *
   * @param \Drupal\Core\Config\Entity\ConfigEntityInterface $entity
   *   The entity being formatted.
   * @param array $form
   *   Reference to the form structure.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The complete form state.
   */
  public function addThirdPartyConfig(ConfigEntityInterface $entity, array &$form, FormStateInterface $form_state): void;

}
